# autograde

`autograde` is a toolbox for testing Jupyter notebooks. Autograde executes notebooks with consecutive unit testing of the final notebook state and allows refining results (e.g. grading plots by hand) in the audit mode. Autograde summarizes results in human (html) and machine-readable (JSON) formats.
https://github.com/cssh-rwth/autograde

## Demo

We have included `autograde` to this envirnment by adding a line `jupyter-autograde==0.2.13` to `requirements.txt` file.
Let's evaluate how it automates grading using the example files provided by `autograde` in the `demo/` subdirectory. 

### Grade
To grade `demo/notebook.ipynb` we run `autograde test` against test scenarios specified in `demo/test.py` using context files stored in `demo/context`
``` bash
autograde test demo/test.py demo/notebook.ipynb --target /grading --context demo/context
```

- `demo/test.py` contains test cases we want to apply
- `demo/notebook.ipynb` is the a notebook to be tested (here you may also specify a directory to be recursively searched for notebooks)
- The optional flag `--target` tells autograde where to store results, /tmp in our case, and the current working directory by default.
- The optional flag `--context` specifies a directory that is mounted into the sandbox and may contain arbitrary files or subdirectories. This is useful when the notebook expects some external files to be present such as data sets.

This command generates a compressed archive named `results_[Lastname1,Lastname2,...]_XXXXXXXX.zip` (in this case `results_[Bar,Foo]_598b9a96.zip`) with the following contents:

- `artifacts/`: directory with all files that where created or modified by the tested notebook as well as rendered matplotlib plots.
- `code.py`: code extracted from the notebook including stdout/stderr as comments
- `notebook.ipynb`: an identical copy of the tested notebook
- `restults.json`: test results

### Inspect and edit automatic grades
Let's use `autograde audit` to inspect and edit the grades generated by automatic testing located in subfolder `grading/`.
```bash
autograde audit grading/
```
  
To open an interactive audit interface triggered by this function we first need to switch to virtual desktop by clicking the VNC icon:  
![image](https://renkulab.io/gitlab/learn-renku/teaching-on-renku/autograde/raw/master/images/toVNC.png)
  
Now we can access the interactive interface running at http://127.0.0.1:5000/   
![image](https://renkulab.io/gitlab/learn-renku/teaching-on-renku/autograde/raw/master/images/audit_interface_480.gif)
  
**Preview** results of automtic grading.  
![image](https://renkulab.io/gitlab/learn-renku/teaching-on-renku/autograde/raw/master/images/preview_results_480.gif)

**Edit** and **comment** the results of automatic grading.  
![image](https://renkulab.io/gitlab/learn-renku/teaching-on-renku/autograde/raw/master/images/edit_results_480.gif)

### Generate Reports
Let's generate a human readable HTML report for grading results using `autograde report`, the command below generates a report inside the results archive `grading/results_\[Bar\,Foo\]_598b9a96.zip`.
```bash
autograde report grading/
```

Let's uncompress the results
```bash
pushd grading/
unzip results_\[Bar\,Foo\]_598b9a96.zip
popd
```
  
Here is how the generated `grading/report.html` looks like:  
![image](https://renkulab.io/gitlab/learn-renku/teaching-on-renku/autograde/raw/master/images/report_html.gif)
You can display the full contents of this report by opening it either from interactive session or locally.

Let's summarise results and generate inside subfolder `grading/` the follwoing 2 files:
- `summary.csv` with aggregated results
- `summary.html` with human readable summary report
```bash
autograde summary grading/
```
  
Here is how the generated summary `grading/summary.html` looks like:  
![image](https://renkulab.io/gitlab/learn-renku/teaching-on-renku/autograde/raw/master/images/summary_html.gif)

